-- Raid Planner
-- Version: 0.0.4
-- Author: Azulorr - US:Caelestrasz
local _, VitaPlanner = ...

local VP_BOSSFRAME_MAXNUM   = 1;
local VP_BOSSFRAME_HEIGHT   = 68;
local VP_BOSSFRAME_PADDING  = 15;

VitaPlanner = LibStub("AceAddon-3.0"):NewAddon(VitaPlanner, "VitaPlanner", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceGUI = LibStub("AceGUI-3.0")

_G.VitaPlanner = VitaPlanner

local _G = _G

local realmName = gsub(gsub(GetRealmName(), ' ', ''), '-', '')

local audioPlayed = false

-- Beta Version 5
local sVersion = "0.5.0"
local iVersion = 5

function VitaPlanner:OnInitialize()
	self.sVersion = sVersion
	self.iVersion = iVersion

	self.db = LibStub("AceDB-3.0"):New("VitaP_DB")
	
	RegisterAddonMessagePrefix("VitaPlannerC")
	RegisterAddonMessagePrefix("VPVerRequest")
	RegisterAddonMessagePrefix("VPVerResponse")
	
	self:RegisterComm("VitaPlannerC",	"CommandReceived")
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED",     "EnterCombat");
    self:RegisterEvent("PLAYER_REGEN_ENABLED",      "LeaveCombat");
	
	self:RegisterComm("VPVerRequest",	"VersionRequest")
    self:RegisterComm("VPVerResponse",	"VersionResponse")
	
	self:SetupOptions()
	
	self.bossList   = {}
	self.bossLCache = {}
	self.frame 		= nil
	
	self.my_set_group = nil
	
	self.timeout = 0
	self.timeoutLeft = 0
	
	self.RESPONSE = {
        { ["CODE"]      = "IN",   	["SORT"] =  100,  ["COLOR"] = {1,1,1},        ["TEXT"] = "In" },
        { ["CODE"]      = "OUT",    ["SORT"] =  200,  ["COLOR"] = {1,0,0},        ["TEXT"] = "Out" }
	}
	
	if IsInGuild() then
		self:SendCommMessage("VPVerRequest",      iVersion .. "_" .. sVersion, "GUILD")
	end
end

function VitaPlanner:OnEnable()

end

function VitaPlanner:OnDisable()
	
end

function VitaPlanner:EnterCombat()
    -- Should we hide when entering combat?
    --if not VitaPlanner.db.profile.hideSelectionOnCombat then return end;

    self.inCombat = true;
    if self.frame and self.frame:IsShown() then
        self.hiddenOnCombat = true;
        self:Hide();
    end
end

function VitaPlanner:LeaveCombat()
    self.inCombat = nil;
    self:UpdateSelectionUI();
end

function VitaPlanner:IsShown()
    if not self.frame then return false end;

    if self.inCombat and self.hiddenOnCombat then
        return true;
    end;

    return self.frame:IsShown();
end

function VitaPlanner:Show()
    if not self.frame then return false end;

	if VitaPlannerL and VitaPlannerL.IsShown then
        if not self.rplshown then
            self.rplshown = VitaPlannerL.IsShown(VitaPlannerL)
        end
        VitaPlannerL.Hide(VitaPlannerL)
    end
	
    if self.inCombat then
        self.hiddenOnCombat = true;
        return self.frame:Hide();
    end
	
    return self.frame:Show();
end

function VitaPlanner:Hide()
    if not self.frame then return end;
	
    local ret = self.frame:Hide();
		
    if not self.inCombat and self.rplshown and VitaPlannerL and VitaPlannerL.Show then
        VitaPlannerL.Show(VitaPlannerL)
        self.rplshown = false;
    end
	
    return ret;
end

local function GetOptions()
	VitaPlanner.RPOptions = {
		name = "VitaPlanner",
		handler = VitaPlanner,
		type = 'group',
		plugins = {},
		args = {
			msg = {
				type = 'input',
				name = 'My Message',
				desc = 'The message for my addon',
			},		
		},
	}
	VitaPlanner.RPOptions.plugins.profiles = { profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(VitaPlanner.db) }
	
	return VitaPlanner.RPOptions
end


-- Unused ATM
function VitaPlanner:Command(input)
	LibStub("AceConfigDialog-3.0"):Open("VitaPlanner")
end

--function RaidPlannner:RegisterOptions(key, table)
--	if not self.RPOptions then
--		error("Options table has not been created yet, respond to the callback!", 2)
--	end
--	self.RPOptions.plugins[key] = { [key] = table }
--end

function VitaPlanner:SetupOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("VitaPlanner", GetOptions)
	AceConfigDialog:SetDefaultSize("VitaPlanner", 880,525)
	--VitaPlanner:RegisterChatCommand("vp", "Command")
end

function VitaPlanner:VersionRequest(prefix, message, distribution, sender)
	local _,_,senderVersion, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	senderVersion = tonumber(senderVersion) or 0
	sender = VitaPlanner.UnAmbiguate(sender)
	
	if (senderVersion<iVersion) then
		-- Senders version has been outdated
		self:SendCommMessage("VPVerResponse", iVersion .. "_" .. sVersion, "WHISPER", sender)
	end
	if (senderVersion>iVersion) then
		-- Our version is outdated
        self:ShowUpdateFrame( sender, iVersion, senderVersionString )
	end
end

function VitaPlanner:VersionResponse(prefix, message, distribution, sender)
	local _,_,senderVersion, senderVersionString = string.find(message, "^(%d+)_(.*)$")
	senderVersion = tonumber(senderVersion) or 0
	sender = VitaPlanner.UnAmbiguate(sender)
	
	--[[if (senderVersion<iVersion) then
		-- Senders version has been outdated
		self:SendCommMessage("VPVerResponse", iVersion .. "_" .. sVersion, "WHISPER", sender)
	end--]]
	if (senderVersion>iVersion) then
		-- Our version is outdated
        self:ShowUpdateFrame( sender, iVersion, senderVersionString )
	end
end

function VitaPlanner:ShowUpdateFrame( sender, iVersion, sVersion )
    if self.iLastVersionResponse and self.iLastVersionResponse>=iVersion then
        -- Only show the update message once, unless there is a newer version found.
        return;
    end;
    self.iLastVersionResponse = iVersion;
    message( string.format("Auto notice from %s: Please update 'Vita Planner' from Curse.com.", sender, sVersion ) )
end

function VitaPlanner:InitUI()
    self.selectionFrame = nil
	self.notificationFrame = nil

    local frame = CreateFrame("Frame","VitaPlannerBossUI",UIParent)
    frame:Hide();
    frame:SetWidth(410)
	frame:SetHeight(160)
    frame:SetPoint("CENTER",UIParent,"CENTER",0,0)
	frame:SetPoint("TOP",UIParent,"TOP",0,-100)
  	frame:EnableMouse()
    frame:SetScale(VitaPlanner.db.profile.popupUIScale or 1)
    frame:SetResizable()
    frame:SetMovable(true)
    frame:SetFrameStrata("DIALOG")
    frame:SetToplevel(true)
    frame:SetBackdrop({
      bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    frame:SetBackdropColor(0.4,0.4,0.4,1)
    frame:SetBackdropBorderColor(1,1,1,0.2)

    frame:SetScript("OnMouseDown", function() frame:StartMoving() end)
    frame:SetScript("OnMouseUp", function() frame:StopMovingOrSizing() end)

    local titleFrame = CreateFrame("Frame", nil, frame)
    titleFrame:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    titleFrame:SetBackdropColor(0,0,0,1)
    titleFrame:SetHeight(22)
    titleFrame:EnableMouse()
    titleFrame:EnableMouseWheel(true)
    titleFrame:SetResizable()
    titleFrame:SetMovable(true)
    titleFrame:SetPoint("LEFT",frame,"TOPLEFT",20,0)
    titleFrame:SetPoint("RIGHT",frame,"TOPRIGHT",-20,0)

    titleFrame:SetScript("OnMouseDown", function() frame:StartMoving() end)
    titleFrame:SetScript("OnMouseUp", function() frame:StopMovingOrSizing() end)
    titleFrame:SetScript("OnMouseWheel", function(s, delta)
      self:SetUIScale( max(min(frame:GetScale(0.8) + delta/15,5.0),0.5) );
    end)

    local titletext = titleFrame:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
    titletext:SetPoint("CENTER",titleFrame,"CENTER",0,1)
	titletext:SetText ( "Vita Planner by Azulorr <Vita Obscura> - Caelestrasz US-Oceanic" )
    frame.titleFrame = titleFrame

    -- Add our frame to the addon globally
    self.frame = frame;
	-- Return the new frame
	return self.frame
end

function VitaPlanner:CreateBossSelectionFrame()
	if not self.frame then self:InitUI() end;

    local frame = CreateFrame("Frame", nil, self.frame)
    frame:Show()
    frame:SetHeight(self.frame:GetHeight() - 10)
    frame:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    frame:SetBackdropColor(0.12,0.12,0.12,0.6)
    frame:SetPoint("LEFT",self.frame,"LEFT",20,0)
    frame:SetPoint("RIGHT",self.frame,"RIGHT",-20,0)
	
	local lblName = CreateFrame("Button", nil, frame)
	lblName:SetNormalFontObject("GameFontNormalLarge")
	lblName:SetHeight(25)
	lblName:SetFrameStrata("TOOLTIP")
	lblName:SetPoint("TOPLEFT",frame,"TOPLEFT",10,-10)
	frame.bossName = lblName	
	
	local lblDifficulty = frame:CreateFontString(nil,"OVERLAY","GameFontNormalLeft")
	lblDifficulty:SetText( "Difficulty:" )
	lblDifficulty:SetPoint( "TOPLEFT", lblName, "BOTTOMLEFT", 0, -10)	

    local lblDiff = frame:CreateFontString(nil,"OVERLAY","GameFontNormalLeft")
    lblDiff:SetPoint("TOPRIGHT",lblDifficulty,"TOPLEFT",200,0)
    lblDiff:SetText( "BossDiff" )
    frame.lblDiff = lblDiff;

    local lblPlanner = frame:CreateFontString(nil,"OVERLAY","GameFontNormalLeft")
    lblPlanner:SetPoint("TOPLEFT",lblDifficulty,"BOTTOMLEFT",0,-10)
    lblPlanner:SetText( "Planner:" )
	frame.lblPlanner = lblPlanner
	
	local planner = frame:CreateFontString(nil,"OVERLAY","GameFontNormalLeft")
	planner:SetPoint("TOPLEFT",lblPlanner,"TOPLEFT",200,0)
	planner:SetText("-")
    frame.planner = planner;

    frame.buttons = {}
	
    --[[local btnPass = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
    btnPass:SetScript("OnClick", function()
              self:SendBossSelection( frame.data.leader, frame.data.bossName, self.RESPONSE.PASS)
              self:RemoveSelection( frame.data.bossName );
              self:UpdateSelectionUI();
    end)
    btnPass:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-10,10)
    btnPass:SetText("Ignore")
    btnPass:SetHeight(25)
    btnPass:SetWidth(70)
    frame.btnPass = btnPass--]]

	--[[
    local timerFrame = CreateFrame("Frame", nil, frame)
    timerFrame:SetHeight(20)
      timerFrame:SetWidth(115);
      timerFrame:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      --edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    timerFrame:SetBackdropColor(1, 0, 0, 0.4)
    timerFrame:SetBackdropBorderColor(1, 0.6980392, 0, 0)
    timerFrame:SetPoint("BOTTOMRIGHT",btnPass,"TOPRIGHT", 0, 10);
    frame.timerFrame = timerFrame;

    local b=CreateFrame("STATUSBAR",nil,timerFrame,"TextStatusBar");
    local bCount = 0;
    local bElapse = 0;
    b:SetPoint("TOPLEFT",timerFrame,"TOPLEFT", 3, -3);
    b:SetPoint("BOTTOMRIGHT",timerFrame,"BOTTOMRIGHT", -2, 3);
    b:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar");
    b:SetStatusBarColor(0.4, 0.8, 0.4, 0.8);
    b:SetMinMaxValues(0,100)
    b:SetScript("OnUpdate", function( o, elapsed )
        if not frame or not frame.data or not frame.data.timeoutLeft then
            b:SetMinMaxValues(0,100); b:SetValue(100);
            frame.lblTimeout:SetText( "No Timeout" )
            return;
        end
        frame.data.timeoutLeft = frame.data.timeoutLeft - elapsed;
        if frame.data.timeoutLeft<0 then
            frame.data.timeoutLeft = 0;
            b:SetValue(0);
            frame.lblTimeout:SetText( "No Time Left" )

            self:SendBossSelection( frame.data.leader, frame.data.bossName, self.RESPONSE.TIMEOUT)
            self:RemoveSelection( frame.data.bossName );
            self:UpdateSelectionUI();

            return;
        end

        frame.lblTimeout:SetText( format("%s secs until pass", ceil(frame.data.timeoutLeft)) )
        b:SetValue(frame.data.timeoutLeft)
    end)
    frame.progressBar = b;

    local timerBorderFrame = CreateFrame("Frame", nil, timerFrame)
    timerBorderFrame:SetHeight(20)
      timerBorderFrame:SetToplevel(true)
      timerBorderFrame:SetWidth(135);
      timerBorderFrame:SetBackdrop({
      --bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    timerBorderFrame:SetBackdropColor(1, 0, 0, 0.0)
    timerBorderFrame:SetBackdropBorderColor(1, 0.6980392, 0, 1)
    timerBorderFrame:SetPoint("TOPLEFT",timerFrame,"TOPLEFT", 0, 0);
    timerBorderFrame:SetPoint("BOTTOMRIGHT",timerFrame,"BOTTOMRIGHT", 0, 0);

    local lblTimeout = timerBorderFrame:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
    lblTimeout:SetPoint("CENTER",timerBorderFrame,"CENTER",0,0)
    lblTimeout:SetVertexColor( 1, 1, 1 );
    lblTimeout:SetText( "Itemname" )
    frame.lblTimeout = lblTimeout;
	--]]
    --timerFrame:Show()
	
	self.checkTimerFrame = self:GenerateTimerFrame()
	self.checkTimerFrame:Hide()
	
    return frame;
end

function VitaPlanner:Debug( message, verbose )
    if not self.debug then return end;
    if verbose and not self.verbose then return end;
    self:Print("debug: " .. message)
end

function VitaPlanner:SendLeaderCommand(command, message, target)
	if not target then
		return self:Print(L["Could not send command, no target specified"])
	end
	
	local formatted = format("%s:%s", tostring(command), tostring(message));
    local broadcasted = false
	
	if VitaPlanner.UnitInRaid(target) then
        self:SendCommMessage("VitaPlannerL", formatted, "RAID", nil, "ALERT")
        self:Debug('SendCommand(RAID): '..formatted, true)
		broadcasted = true
	elseif VitaPlanner.UnitInParty(target) and GetNumSubgroupMembers()>0 then
        self:SendCommMessage("VitaPlannerL", formatted, "PARTY", nil, "ALERT")
        self:Debug('SendCommand(PARTY): '..formatted, true)
        broadcasted = true;
    else
		target = VitaPlanner.Ambiguate(target, "none")
        self:SendCommMessage("VitaPlannerL", formatted, "WHISPER", target, "ALERT")
        self:Debug('SendCommand(WHISPER->'..target..'): '..formatted, true)
    end
end

function VitaPlanner:UpdateSelectionUI()
	local hasSelections = false
    local totalSelections = 0

    --if self.inCombat then
    --    self.hiddenOnCombat = true
     --   self:Hide()
    --    return true;
    --end

    if not self.frame then
        self:InitUI()
    end;

    totalSelections = #self.bossList;
    hasSelections = totalSelections>0

    for i, data in ipairs(self.bossList) do repeat
        -- If the item is empty try the next item.
        if not data then break end

        local selectionFrame = self.selectionFrame
        if not selectionFrame then			
			selectionFrame = self:CreateBossSelectionFrame()
			self.selectionFrame = selectionFrame
		end
        --end

        selectionFrame.data = data;

		local journalLink = "|cff66bbff|Hjournal:1:" .. data.bossID .. ":" .. data.diffID .. "|h["..data.bossName.."]|h|r"        
		
		selectionFrame.lblDiff:SetText(format('%s',data.diffName))
		local diffLabel = selectionFrame.lblDiff
		
		selectionFrame.bossName:SetText(format('%s',journalLink))
		selectionFrame.bossName:SetWidth(selectionFrame.bossName:GetFontString():GetStringWidth() + 20)
		selectionFrame.bossName:SetScript("OnClick", function()
				--HandleModifiedItemClick( journalLink )
				if ( not EncounterJournal ) then
					EncounterJournal_LoadUI();
				end
				if EncounterJournal:IsShown() then
					EncounterJournal_ListInstances()
					EncounterJournal_DisplayInstance(data.raidID)
					EncounterJournal_DisplayEncounter(data.bossID)
					EJ_SetDifficulty(data.diffID)
				else
					ToggleEncounterJournal()
					EncounterJournal_ListInstances()
					EncounterJournal_DisplayInstance(data.raidID)
					EncounterJournal_DisplayEncounter(data.bossID)
					EJ_SetDifficulty(data.diffID)
				end
		end)

        -- Create a starting anchor for the buttons
        local lastButton = nil
		local totalButtonWidth = 0

		-- Create / Display the buttons
        for i=1, data.numButtons do
			local buttonData = data.buttons[i]
			local button = selectionFrame.buttons[i]

			if not button then
				-- Create the button
				button = CreateFrame("Button", nil, selectionFrame, "UIPanelButtonTemplate")
				button:SetScript("OnClick", function(btn)
					--print("Clicked "..buttonData.text)
					--print("Sending to "..selectionFrame.data.leader..": "..selectionFrame.data.bossName.." - "..buttonData.text)
					self:SendBossSelection(selectionFrame.data.leader, selectionFrame.data.bossName, buttonData.response)
					--print("Removing "..selectionFrame.data.bossName)
					self:RemoveSelection(selectionFrame.data.bossName)
					self:UpdateSelectionUI()
				end)
				button:SetHeight(25)
				selectionFrame.buttons[i] = button
			end

			button:Show()
			button.response = buttonData.response
			button:ClearAllPoints()			
			if not lastButton then
				button:SetPoint("BOTTOMLEFT", selectionFrame, "BOTTOMLEFT", 10, 10)
			else
				button:SetPoint("BOTTOMLEFT", lastButton, "BOTTOMRIGHT", 10, 0)
			end

			button:SetText(buttonData.text)
			local width = button:GetFontString():GetStringWidth()
			width = width + width/20 + 30
			button:SetWidth(width)
			totalButtonWidth = totalButtonWidth + width + 5

			lastButton = button
        end

        -- Position the pass button
        --selectionFrame.btnPass:ClearAllPoints()
        --selectionFrame.btnPass:SetPoint("BOTTOMLEFT", lastButton, "BOTTOMRIGHT", 10, 0)

        -- Hide unused buttons
        for i=data.numButtons + 1, #(selectionFrame.buttons) do
			local button = selectionFrame.buttons[i]
			button:Hide()
        end
        
        selectionFrame.planner:SetText(format("%s", data.leader or "Unknown"))
		selectionFrame.planner:SetPoint("TOPLEFT",diffLabel,"BOTTOMLEFT",0,-10)
		
		self.timeout = data.timeout
		self.timeoutLeft = data.timeoutLeft
        if data.timeout and data.timeout>0 then
            self.checkTimerFrame.progressBar:SetMinMaxValues(0,self.timeout)
            self.checkTimerFrame.progressBar:SetValue(self.timeoutLeft)
            self.checkTimerFrame.lblTimeout:SetText('')
            self.checkTimerFrame:Show()
        else
            self.checkTimerFrame:Hide()
        end
        
		if not self.notificationFrame then
			self.notificationFrame = self:CreateNotificationFrame()
		end
		self.notificationFrame:Hide()
		selectionFrame:Show()
		
    until true end

    if hasSelections then
        self:Show()
        if (not audioPlayed) then --and VitaPlanner.db.profile.audioWarningOnSelection then
            PlaySoundFile("Sound\\Creature\\BabyMurloc\\BabyMurlocA.wav")
            audioPlayed = true
        end
    else
        audioPlayed = false
        self:Hide();
    end

end

function VitaPlanner:CreateNotificationFrame()
	if not self.frame then
        self:InitUI()
    end;

	local frame = CreateFrame("Frame", nil, self.frame)
    frame:Show()
    frame:SetHeight(self.frame:GetHeight() - 10)
	frame:SetWidth(self.frame:GetWidth() - 20)
    frame:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    frame:SetBackdropColor(0.12,0.12,0.12,0.6)
    frame:SetPoint("LEFT",self.frame,"LEFT",20,0)
    frame:SetPoint("RIGHT",self.frame,"RIGHT",-20,0)
	
	local bossName = frame:CreateFontString(nil,"OVERLAY","GameFontNormalLarge")
	bossName:SetText("BossName")
	bossName:SetPoint("TOP", frame, "TOP", 0, -20)
	frame.bossName = bossName
	
	local setMessage = frame:CreateFontString(nil,"OVERLAY","GameFontNormalLarge")
	setMessage:SetFont("Fonts\\FRIZQT__.TTF", 30)
	setMessage:SetPoint("CENTER", frame, "CENTER", 0, 0)
	frame.setMessage = setMessage
	
	local closeBtn = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
	closeBtn:SetText("Close")
	closeBtn:SetWidth(closeBtn:GetFontString():GetWidth() + 20)
	closeBtn:SetPoint("BOTTOM",frame,"BOTTOM",0,20)
	closeBtn:SetScript("OnClick", function()
		self:Hide()
	end)
	
	return frame
	
end

function VitaPlanner:NotifySetGroup()
	if not self.frame then
        self:InitUI()
    end;
	
	local notificationFrame = self.notificationFrame
	if not notificationFrame then
		notificationFrame = self:CreateNotificationFrame()
		self.notificationFrame = notificationFrame
	end
	
	notificationFrame.bossName:SetText(self.set_boss_name)
	
	local isIn = nil
	if tonumber(self.my_set_group) <= tonumber(self.max_set_group) then
		isIn = true
		notificationFrame.setMessage:SetText("YOU ARE IN 1-"..self.max_set_group)
		notificationFrame:SetBackdropColor(0,0.5,0,0.6)
	else
		notificationFrame.setMessage:SetText("YOU ARE OUT")
		notificationFrame:SetBackdropColor(0.5,0,0,0.6)
	end
	
	self.selectionFrame:Hide()
	notificationFrame:Show()
	self:Show()
	
	if isIn then
		PlaySoundFile("Sound\\Interface\\iquestcomplete.ogg")
	else
		PlaySoundFile("Sound\\Interface\\igquestfailed.ogg")
	end
end

function VitaPlanner:RemoveSelection( bossName )
    local index = self:HasBossMessage(bossName)
    if not index then return false end
    tremove(self.bossList, index)
    return true
end

function VitaPlanner:CommandReceived(prefix, message, distribution, sender)
	local _,_,command, message = string.find(message, "^([%a_]-):(.*)$")
	command = strupper(command or '');
	message = message or '';

	sender = VitaPlanner.UnAmbiguate(sender)
	
	if command == "CHECK_BOSS" then
		self.my_set_group = nil
		-- Split message into individual variables
		local raidID, bossName, bossID, diffName, diffID, numButtons, buttons, timeout = strsplit("^", message)
		
		-- Get buttons and number of buttons from message
		numButtons = tonumber(numButtons or 0)
		local buttonsOK = false
		if numButtons >= 1 and buttons and buttons~='' then
			buttons = {strsplit("*", buttons)}
			if #buttons == numButtons then
				for i=1,numButtons do
					local bResponse, bText, bColour = strsplit(';', buttons[i])
					buttons[i] = {
						response                = bResponse or 'NONE',
						text                    = bText or '[empty]',
						colour                   = bColour or 'ffffff'
					}
				end
				buttonsOK = true
			end
		end	

		if not buttonsOK then
			-- Create the default buttons
			numButtons = 2
			buttons = {
				{response = "IN",           text = "In",	colour = 'ffffff'},
				{response = "OUT",          text = "Out",	colour = 'ffffff'}
			}
		end	
		
		if not self:HasBossMessage(bossName) then
			-- add the loot to the bosslist and redraw the ui
			tinsert( self.bossList, {
				["leader"]      	= sender,
				["raidID"]			= raidID,
				["bossName"]        = bossName,
				["bossID"]			= bossID,
				["diffName"]   		= diffName,
				["diffID"]			= diffID,
				["timeout"]         = tonumber(timeout),
				["timeoutLeft"]     = tonumber(timeout),
				["buttons"]         = buttons,
				["numButtons"]      = numButtons
			})

			self:UpdateSelectionUI();
		end		
	elseif command == "SET_GROUPS" then
		local setBoss, maxInGroup, raidersList = strsplit("^",message)
		local raiderList = { strsplit("*",raidersList) }
		for index,raider in ipairs(raiderList) do
			local name, group = strsplit(";", raider)
			if VitaPlanner.UnAmbiguate(name) == VitaPlanner.UnAmbiguate(UnitName("player")) then
				self.my_set_group = group
				self.set_boss_name = setBoss
				self.max_set_group = maxInGroup
				self:NotifySetGroup()
			end
		end
	end		
end

function VitaPlanner:HasBossMessage( bossName )
    for i, data in ipairs(self.bossList) do repeat
        if not data then break end
        if data.bossName == bossName then
            return i
        end
    until true end
    return nil
end

function VitaPlanner:SendBossSelection( target, bossID, response )
    -- Just whisper the response back to the leader.
	--if not response then
	--	response = '0'
	--end
	--print(target .. " - " .. bossID .. " - " .. response)
	if response == "" then
		response = "NONE"
	end
    self:SendLeaderCommand("SELECTION", format("%s^%s", bossID or 0, response), target)
end

function VitaPlanner:GenerateTimerFrame()
	--if VitaPlannerL and VitaPlannerL.checkTimerFrame then
	--	print("Planner timer found, returning that frame:")
	--	return VitaPlannerL.checkTimerFrame
	--end
	--if not self.frame then self:InitUI() end
	-- Timer
	local timerFrame = CreateFrame("Frame", nil, UIParent)
	timerFrame:EnableMouse()
	timerFrame:SetMovable(true)
    timerFrame:SetHeight(30)
	timerFrame:SetWidth(150);
	timerFrame:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		--edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 64, edgeSize = 12,
		insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    timerFrame:SetBackdropColor(1,0,0,0.4)
    timerFrame:SetBackdropBorderColor(1, 0.6980392, 0, 0)
    timerFrame:SetPoint("TOP",UIParent,"TOP", 0, -50);
	
	-- Create our title frame
    local timerTitle = CreateFrame("Frame", nil, timerFrame)
    --#region Setup main frame title
    timerTitle:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    timerTitle:SetBackdropColor(0,0,0,1)
    timerTitle:SetHeight(22)
    timerTitle:EnableMouse()
    timerTitle:EnableMouseWheel(true)
    timerTitle:SetResizable()
    timerTitle:SetMovable(true)
    timerTitle:SetPoint("LEFT",timerFrame,"TOPLEFT",0,10)
    timerTitle:SetPoint("RIGHT",timerFrame,"TOPRIGHT",0,10)

    timerTitle:SetScript("OnMouseDown", function() timerFrame:StartMoving() end)
    timerTitle:SetScript("OnMouseUp", function() timerFrame:StopMovingOrSizing() end)
    timerTitle:SetScript("OnMouseWheel", function(s, delta)
      self:SetUIScale( max(min(timerFrame:GetScale(0.8) + delta/15,5.0),0.5) );
    end)
	
	-- Create the text to go into the title frame
    local titletext = timerTitle:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")    
	titletext:SetText ( "Vita Planner Check Timer" )
    timerFrame.timerTitle = timerTitle
	titletext:SetPoint("CENTER",timerTitle,"CENTER",0,0)

    local b=CreateFrame("STATUSBAR",nil,timerFrame,"TextStatusBar");
    local bCount = 0;
    local bElapse = 0;
    b:SetPoint("TOPLEFT",timerFrame,"TOPLEFT", 3, -3);
    b:SetPoint("BOTTOMRIGHT",timerFrame,"BOTTOMRIGHT", -2, 3);
    b:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar");
    b:SetStatusBarColor(0.4, 0.8, 0.4, 0.8);
    b:SetMinMaxValues(0,100)
    b:SetScript("OnUpdate", function( o, elapsed )
		--print("timerUpdate")
        if not timerFrame or not self.timeoutLeft then
			--print("no timeout found")
		
            b:SetMinMaxValues(0,100); b:SetValue(100);
            timerFrame.lblTimeout:SetText( "No Timeout" )
			if VitaPlannerL and VitaPlannerL.cancelCheckRaid then
				--print("found VPL cancel, hiding it")
				VitaPlannerL.currentlyChecking = true
				VitaPlannerL.cancelCheckRaid:Hide()
			end
			--print("hiding this timer")
			self.checkTimerFrame:Hide()
            return;
        end
        self.timeoutLeft = self.timeoutLeft - elapsed;
        if self.timeoutLeft<0 then
            self.timeoutLeft = 0;
            b:SetValue(0);
            timerFrame.lblTimeout:SetText( "No Time Left" )
			if VitaPlannerL then
				if not VitaPlannerL.selectedBossList and VitaPlannerL.mainFrame.bossInfo then
					VitaPlannerL.mainFrame.bossInfo.checkRaid:Disable()
				elseif VitaPlannerL.mainFrame.bossInfo then
					VitaPlannerL.mainFrame.bossInfo.checkRaid:Enable()
				end				
				if VitaPlannerL.cancelCheckRaid then
					VitaPlannerL.cancelCheckRaid:Hide()
				end
				VitaPlannerL.currentlyChecking = false
			end
			
			if self:IsShown() then
				self:SendBossSelection( self.selectionFrame.data.leader, self.selectionFrame.data.bossName, "NONE")
				self:RemoveSelection( self.selectionFrame.data.bossName );
				self:UpdateSelectionUI()
			end

			self.checkTimerFrame:Hide()
            return;
        end
		
		
        timerFrame.lblTimeout:SetText( format("%s secs left", ceil(self.timeoutLeft)) )
        b:SetValue(self.timeoutLeft)
		if VitaPlannerL and VitaPlannerL.mainFrame then
			VitaPlannerL.currentlyChecking = true
			VitaPlannerL.mainFrame.bossInfo.checkRaid:Disable()
			VitaPlannerL.cancelCheckRaid:Show()
		end
		--self.mainFrame.bossInfo.checkRaid:Disable()
		--self.cancelCheckRaid:Show()
    end)
    timerFrame.progressBar = b;

    local timerBorderFrame = CreateFrame("Frame", nil, timerFrame)
    timerBorderFrame:SetHeight(timerFrame:GetHeight())
	timerBorderFrame:SetToplevel(true)
	timerBorderFrame:SetWidth(timerFrame:GetWidth() + 20);
	timerBorderFrame:SetBackdrop({
		--bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 64, edgeSize = 12,
		insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    timerBorderFrame:SetBackdropColor(1, 0, 0, 0.0)
    timerBorderFrame:SetBackdropBorderColor(1, 0.6980392, 0, 1)
    timerBorderFrame:SetPoint("TOPLEFT",timerFrame,"TOPLEFT", 0, 0);
    timerBorderFrame:SetPoint("BOTTOMRIGHT",timerFrame,"BOTTOMRIGHT", 0, 0);

    local lblTimeout = timerBorderFrame:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
    lblTimeout:SetPoint("CENTER",timerBorderFrame,"CENTER",0,0)
    lblTimeout:SetVertexColor( 1, 1, 1 );
    lblTimeout:SetText( "Itemname" )
    timerFrame.lblTimeout = lblTimeout;
	
	return timerFrame	
end

-- Ambiguate and Unit in X functions copied from EPGP Lootmaster
function VitaPlanner.UnAmbiguate(name)
	-- Don't postfix these default values
	if name == '' or name == nil or name == 'RAID' or name == 'PARTY' or name == 'BATTLEGROUND' or name == 'GUILD' then
		return name
	end

	-- Let the wow client try to add the realmname
	-- This function is really buggy by the way... If realm name has space in it,
	-- it expects you to deliver it to this function without the spaces. /facepalm
	-- So Mackatack-Lightning's Hammer just returns the same, Mackatack-Lightning'sHammer
	-- does only return Mackatack, as is expected. The quote is preserved, just the space is
	-- removed.
	-- Other functions, such as SendAddonMessage and SendChatMessage do work with the space removed
	-- or with the spaces intact. UnitIsUnit etc still only work without the server names. Now quite
	-- Sure how to properly use these functions cross-realm
	local res = VitaPlanner.Ambiguate(name, "none")

	-- No realmname on the playerName? add it
	if strfind(res, "-", nil, true) == nil then
		-- add the realmname to the player name
		res = res .. "-" .. realmName
	end

	return res
end

-- Just a local cache of the Ambiguate function
-- This function currently is broken if there are spaces in the
-- player/realmName. It's easily detected when the blizz devs have fixed this
-- So leave a message to the player when they have so they can report back
local ambiguateSpacesBroken = true
if strfind(GetRealmName(), ' ', nil, true) ~= nil then
	-- We're on a server with a space in the realmName, see if blizzard has fixed the error
	if Ambiguate(UnitName("player").."-"..GetRealmName(), "none") == UnitName("player") then
		VitaPlanner:Print("NOTICE! Ambiguate spaces error has been fixed by Blizzard, please report back to VitaPlanner developer!")
		-- Auto unpatch
		ambiguateSpacesBroken = false
		realmName = GetRealmName()
		-- Else, still broken
	end
end

function VitaPlanner.Ambiguate(name, aType, ...)
	-- It seems this function totally bugs out when there are spaces in the player name
	-- So, remove the spaces
	-- Edit: it's even more broken, it doesn't work when the realmName has spaces AND dashes, () or any other UTF char works,
	-- just not spaces and dashes...
	if ambiguateSpacesBroken and name ~= nil and strfind(name, "-", nil, true) ~= nil then
		-- first, just remove the spaces
		-- Then split at the first dash, remove any dashes from the
		-- realmName and concat back together, afterwards, feed this into Ambiguate.
		-- Sweet... bunch of awesome developers they have at Blizz </sarcasm>
		name = gsub(name, ' ', '')
		local n, s = strsplit('-', name, 2)
		name = n .. '-' .. gsub(s, '-', '')
	end
	return Ambiguate(name, aType, ...)
end

-- Don't trust blizzard functions to remove the realmname, just split on dash and return the first value
function VitaPlanner.StripServerName(name)
	local n, s = strsplit('-', name, 2)
	return n
end

function VitaPlanner.UnitInRaid(unit)
	-- Dont do anything special if there's no - in the unit name
	if strfind(unit, "-", nil, true) ~= nil then
		unit = VitaPlanner.Ambiguate(unit, "none")
	end
	return UnitInRaid(unit)
end

-- These built-in wow functions dont work properly with UnAmbiguated names...
function VitaPlanner.UnitInParty(unit)
	-- Dont do anything special if there's no - in the unit name
	if strfind(unit, "-", nil, true) ~= nil then
		unit = VitaPlanner.Ambiguate(unit, "none")
	end
	return UnitInParty(unit)
end

-- These built-in wow functions dont work properly with UnAmbiguated names...
-- for example UnitIsUnit("Bushmaster", "player") return true, UnitIsUnit("Bushmaster-Darksorrow", "player") returns false...
function VitaPlanner.UnitIsUnit(unit1, unit2)
	-- Dont do anything special if there's no - in the unit name
	if strfind(unit1, "-", nil, true) ~= nil then
		unit1 = VitaPlanner.Ambiguate(unit1, "none")
	end
	if strfind(unit2, "-", nil, true) ~= nil then
		unit2 = VitaPlanner.Ambiguate(unit2, "none")
	end
	return UnitIsUnit(unit1, unit2)
end

-- UnitName receives a 2nd return value containing the realm name
function VitaPlanner.UnitName(unit)
	local name, realm = UnitName(unit)
	if name == nil then
		return nil, nil
	end
	if realm ~= nil then
		return name .. "-" .. realm
	end
	return name .. '-' .. realmName
end