-- Raid Planner
-- Version: 5.4.8:03
-- Author: Azulorr - US:Caelestrasz
local _, VitaPlannerL = ...

local RP_BOSSFRAME_MAXNUM   = 4
local RP_BOSSFRAME_HEIGHT   = 68
local RP_BOSSFRAME_PADDING  = 15
local RAID_UNIT_HEIGHT 		= 25
local RAID_UNIT_WIDTH 		= 100
local RAID_UNIT_VERT_GAP 	= 2
local RAID_GROUP_HORZ_GAP 	= 10
local RAID_GROUP_VERT_GAP 	= 40
local MSG_Prefix    		= '[Vita Planner] '

VitaPlannerL 		= LibStub("AceAddon-3.0"):NewAddon(VitaPlannerL, "VitaPlannerL", "AceConsole-3.0", "AceEvent-3.0", "AceComm-3.0", "AceTimer-3.0")
local VitaPlanner 	= LibStub("AceAddon-3.0"):GetAddon("VitaPlanner")

local AceConfigDialog = LibStub("AceConfigDialog-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local LGIST = LibStub("LibGroupInSpecT-1.0")

_G.VitaPlannerL = VitaPlannerL

local _G = _G

local realmName = gsub(gsub(GetRealmName(), ' ', ''), '-', '')

local lastMsgID = nil
local lastMsgFiltered = false

local chatCommands = {
    --'^%s*![iI][nN]%s+(%a+)%s*(.*)',
    '^%s*#([iI][nN])%s*(.*)',
    '^%s*#([oO][uU][tT])%s*(.*)',
    '^%s*#([cC][aA][nN][cC][eE][lL])%s*(.*)',
	'^%s*([%[%]]Vita(%s?)Planner[%[%]])%s*(.*)'
}
local numChatCommands = #chatCommands

local maxSetNum = 6

function VitaPlannerL:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("VitaPL_DB")
	self.Raiders = {}
	
	RegisterAddonMessagePrefix("VitaPlannerL")
	
	self:RegisterComm("VitaPlannerL",	"CommandReceived")
	
	self:RegisterEvent("PLAYER_REGEN_DISABLED",     "EnterCombat");
    self:RegisterEvent("PLAYER_REGEN_ENABLED",      "LeaveCombat");
	
	self:RegisterEvent("GROUP_ROSTER_UPDATE",			"RAID_GROUP_UPDATE");
	self:RegisterEvent("RAID_ROSTER_UPDATE",            "RAID_GROUP_UPDATE");
    self:RegisterEvent("PARTY_LOOT_METHOD_CHANGED",     "RAID_GROUP_UPDATE");
    self:RegisterEvent("PARTY_MEMBERS_CHANGED",         "RAID_GROUP_UPDATE");
    self:RegisterEvent("PLAYER_ENTERING_WORLD",         "RAID_GROUP_UPDATE");
	
	LGIST.RegisterCallback(VitaPlannerL, "GroupInSpecT_Update", "InspectUpdateHandler")
	LGIST.RegisterCallback(VitaPlannerL, "GroupInSpecT_Remove", "InspectRemoveHandler")
	
	--self:SetupOptions()
	
	self.RESPONSE = {
        { ["CODE"]      = "IN",   	["SORT"] =  100,  ["COLOR"] = {1,1,1},        ["TEXT"] = "In" },
        { ["CODE"]      = "OUT",    ["SORT"] =  200,  ["COLOR"] = {1,0,0},        ["TEXT"] = "Out" }
	}
	
	self.BOSSES = {
		{	["INSTANCEID"] = 369, ["ABBR"] = "SoO", ["NAME"] = "Siege of Orgrimmar", ["TIER"] = "T16",
			["BOSS"] = {
				{	["ENCOUNTERID"] =	852,	["NAME"] = "Immerseus"					},
				{	["ENCOUNTERID"] =	849,	["NAME"] = "The Fallen Protectors"		},
				{	["ENCOUNTERID"] =	866,	["NAME"] = "Norushen"					},
				{	["ENCOUNTERID"] =	867,	["NAME"] = "Sha of Pride"				},
				{	["ENCOUNTERID"] =	868,	["NAME"] = "Galakras"					},
				{	["ENCOUNTERID"] =	864,	["NAME"] = "Iron Juggernaut"			},
				{	["ENCOUNTERID"] =	856,	["NAME"] = "Kor'kron Dark Shaman"		},
				{	["ENCOUNTERID"] =	850,	["NAME"] = "General Nazgrim"			},
				{	["ENCOUNTERID"] =	846,	["NAME"] = "Malkorok"					},
				{	["ENCOUNTERID"] =	870,	["NAME"] = "Spoils of Pandaria"			},
				{	["ENCOUNTERID"] =	851,	["NAME"] = "Thok the Bloodthirsty"		},
				{	["ENCOUNTERID"] =	865,	["NAME"] = "Siegecrafter Blackfuse"		},
				{	["ENCOUNTERID"] =	853,	["NAME"] = "Paragons of the Klaxxi"		},
				{	["ENCOUNTERID"] =	869,	["NAME"] = "Garrosh Hellscream"			}
			}
		}
	}
	self.BOSSOPTIONS = {}
	self.BOSSBYABBR = {}
	for i,t in ipairs(self.BOSSES) do
		self.BOSSOPTIONS[t.ABBR] = t.NAME
		self.BOSSBYABBR[t.ABBR] = {
			["NAME"] = t.NAME,
			["ID"] = t.INSTANCEID,
			["LIST"] = t.BOSS
		}
	end
	VitaPlannerL.BOSSOPTIONS = self.BOSSOPTIONS
	VitaPlannerL.BOSSESS = self.BOSSES
	
	-- Difficulties for Selection, Passing to Raid Member, and for Dungeon Journal
	self.DIFFICULTIES = {
		{	["NAME"] = "Flexible",			["VALUE"] = 14 },
		{	["NAME"] = "10 Normal",		["VALUE"] = 3 },
		{	["NAME"] = "25 Normal",		["VALUE"] = 4 },
		{	["NAME"] = "10 Heroic",		["VALUE"] = 5 },
		{	["NAME"] = "25 Heroic",		["VALUE"] = 6 }
		--[[ TO IMPLEMENT IN 6.0
		{	"NAME" = "Normal",	"VALUE" = ? }
		{	"NAME" = "Heroic",	"VALUE" = ? }
		{	"NAME" = "Mythic",	"VALUE" = ? }
		--]]
	}
	self.DIFFOPTIONS = {}
	for i,d in ipairs(self.DIFFICULTIES) do
		self.DIFFOPTIONS[d.VALUE] = d.NAME
	end
	VitaPlannerL.DIFFOPTIONS = self.DIFFOPTIONS
	
	--print (self.BOSSES.T16_LIST[1].NAME)
	
	self.db:RegisterDefaults({
		profile = {					
			selection_timeout = 60,
			selected_raid = "SoO",
			selected_difficulty = 3,
			hideLOnCombat = true
		}
    })
	
	VitaPlannerL:SetupOptions()
	
	self:ScheduleTimer("SetupVariables", 10)
	self:ScheduleTimer("PostEnable", 10)
end

function VitaPlannerL:OnEnable()

end

function VitaPlannerL:OnDisable()
	
end

function VitaPlannerL:PostEnable()
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER",			VitaPlannerL.ChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID",    		VitaPlannerL.ChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_RAID_LEADER",		VitaPlannerL.ChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL",			VitaPlannerL.ChatFilter)
	ChatFrame_AddMessageEventFilter("CHAT_MSG_WHISPER_INFORM", 	VitaPlannerL.ChatFilter)
end

function VitaPlannerL:SetupVariables()
	self.difficulty = self.db.profile.selected_difficulty
	self.diffName = self.DIFFOPTIONS[self.difficulty]
	self.selectedRaid = self.db.profile.selected_raid
	self.selectedBossList = self.BOSSBYABBR[self.selectedRaid].LIST	
	self.zoneName = self.BOSSBYABBR[self.selectedRaid].NAME
	self.zone = self.BOSSBYABBR[self.selectedRaid].ID
	self.selectedBoss = nil
	self.selectedBossID = nil
	
	self.inList = {}
	self.outList = {}
	self.ignoreList = {}
	
	self.timeout = 0
	self.timeoutLeft = 0
	
	self.currentlyChecking = false
	
	self.currentDraggedUnitFrame = nil
end

local function GetOptions()
	VitaPlannerL.RPOptions = {
		name = "VitaPlannerL",
		handler = VitaPlannerL,
		type = 'group',
		get = function(i) return VitaPlannerL.db.profile[i[#i]] end,
		set = function(i, v) VitaPlannerL.db.profile[i[#i]] = v end,
		plugins = {},
		args = {
			global = {
				order = 1,
				type = "group",
				hidden = function(info) return not VitaPlannerL end,
				name = "General Config",
				args = {
					selection_timeout = {
						order = 1,
						type = "select",
						width = "double",
						name = "Boss Selection Timeout",
						desc = "Sets the amount of time a raid member has to select a boss.",
						values = {
							[0] = 'No timeout',
							[10] = '10 secs',
							[15] = '15 secs',
							[20] = '20 secs',
							[30] = '30 secs',
							[40] = '40 secs',
							[50] = '50 secs',
							[60] = '1 minute',
							[90] = '1 min 30 sec',
							[150] = '2 min 30 sec',
							[300] = '5 min',
						}
					},
					selected_raid = {
						order = 2,
						type = "select",
						width = "double",
						name = "Default Selected Raid",
						desc = "Sets the default raid for the planner",
						values = VitaPlannerL.BOSSOPTIONS--[[{
							["SoO"] = "Siege of Orgrimmar"
						}--]]
					},
					selected_difficulty = {
						order = 3,
						type = "select",
						width = "double",
						name = "Default Selected Difficulty",
						desc = "Sets the default difficulty level for the raid",
						values = VitaPlannerL.DIFFOPTIONS --[[{
							[14] = "Flexible",
							[3] = "10 Normal",
							[4] = "25 Normal",					
							[5] = "10 Heroic",
							[6] = "25 Heroic",
						}--]]
					}
				}
			},
		},
	}
	VitaPlannerL.RPOptions.plugins.profiles = { profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(VitaPlannerL.db) }
	
	return VitaPlannerL.RPOptions
end

function VitaPlannerL:EnterCombat()	
	-- On entering combat, we need to hide the Leader frame and set a variable true if in the middle of a check
	self.inCombat = true
	if self:IsShown() then
		-- Hide the frame
		self:Hide()
		-- If in check, set variable to true
		self.isChecking = true
	end
end

function VitaPlannerL:LeaveCombat()
	-- On leaving combat, do the reverse of entering
    self.inCombat = false;
	-- If was not in middle of check then don't reopen
	if not self.isChecking then return end	
	-- Reopen frame
	--self.isChecking = false
	if IsInRaid() and (UnitIsGroupAssistant("player") or UnitIsGroupLeader("player")) then	
		--if VitaPlanner.timeoutLeft > 0 then
			self:Show()
	end
end

function VitaPlannerL:IsShown()
	-- Find out if our addon is shown
	-- If the base frame doesn't exist, then our addon isn't showing
    if not self.frame then return false end;
	-- If we're in combat, and we've told the addon to hide in combat then return true
    if self.inCombat and self.setHiddenOnCombat then return true end
	-- Return the visibility status of the inner frame
    return self.frame:IsShown();
end

function VitaPlannerL:Show()
	-- If we don't have an inner frame then we can't show anything
    if not self.frame then return false end;

	-- If we're not in combat
	if not self.inCombat then
		-- If we're by ourselves, or if we're in a raid and have proper status
		if (IsInRaid() and (UnitIsGroupAssistant("player") or UnitIsGroupLeader("player"))) then	
			-- Show the inner frames
			return self.frame:Show()
		end
	end
	return
end

function VitaPlannerL:Hide()
	-- If we don't have an inner frames then we don't need to hide anything
    if not self.frame then return end;
	-- Hide the inner frames
    local ret = self.frame:Hide();
	-- Return the result
    return ret;
end

function VitaPlannerL:Command(input)
	if input == "options" then
		-- Open the options menu
		LibStub("AceConfigDialog-3.0"):Open("VitaPlannerL")
	elseif not IsInRaid() or (IsInRaid() and (UnitIsGroupAssistant("player") or UnitIsGroupLeader("player"))) then
		-- If we're by ourselves, or if we're in a raid and have proper status, set out frames and variables
		
		-- If we had the main frame before, then continue on, else make a new one
		local pFrame = self.mainFrame
        if not pFrame then
            pFrame = self:CreatePlannerLeaderFrame()			
            self.mainFrame = pFrame
        end
		
		-- If we've never selected a boss, then disable the check raid button
		
		self:UpdateMaxGroupSet()
		-- Update our preference lists
		self:UpdateLists()
		
		-- Show the inner frames
		pFrame:Show()
		-- Show the outer frame
		self:Show()
	else
		-- If you don't have permissions, you can't view the planner
		print("You are not allowed to view this")
	end
end

function VitaPlannerL:RegisterOptions(key, table)
	if not self.RPOptions then
		error("Options table has not been created yet, respond to the callback!", 2)
	end
	self.RPOptions.plugins[key] = { [key] = table }
end

function VitaPlannerL:SetupOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("VitaPlannerL", GetOptions)
	AceConfigDialog:SetDefaultSize("VitaPlannerL", 880,525)
	VitaPlannerL:RegisterChatCommand("vpl", "Command")
end

function VitaPlannerL:InspectUpdateHandler(event, guid, unit, info)
	if info.class and info.spec_role_detailed then
		local unitID = LGIST:GuidToUnit(guid)
		self.Raiders[UnitName(unit)] = {
			["Class"] = 	strupper(info.class),
			["SpecRole"] = 	strupper(info.spec_role_detailed)
		}
		--print (self.Raiders[unitID].Name .. " - " .. self.Raiders[unitID].SpecRole)
	end
end

function VitaPlannerL:InspectRemoveHandler(event, guid)
	local unitID = LGIST:GuidToUnit(guid)
	--print(unitID .. " removed")
	self.Raiders[UnitName(unitID)] = nil
end

function VitaPlannerL:InitUI()
	-- Create our main frame for the planner
	-- This is done only once per session or until the UI is reloaded
    self.mainFrame = nil;
	self.timerFrame = nil

	-- Create our large frame (it acts as a background)
    local frame = CreateFrame("Frame","VitaPlannerLeaderBossUI",UIParent)
    --#region Setup main masterlooter frame
    frame:Hide();
    frame:SetWidth(620)
    frame:SetHeight(560)
    frame:SetPoint("CENTER",UIParent,"CENTER",0,0)
    --frame:SetPoint("TOP",UIParent,"CENTER",0,RP_BOSSFRAME_MAXNUM*(RP_BOSSFRAME_PADDING+RP_BOSSFRAME_HEIGHT)/2)
  	frame:EnableMouse()
    frame:SetScale(VitaPlanner.db.profile.popupUIScale or 1)
    frame:SetResizable()
    frame:SetMovable(true)
    frame:SetFrameStrata("DIALOG")
    frame:SetToplevel(true)
    frame:SetBackdrop({
      bgFile = "Interface\\DialogFrame\\UI-DialogBox-Gold-Background",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    frame:SetBackdropColor(0.4,0.4,0.4,1)
    frame:SetBackdropBorderColor(1,1,1,0.2)

    --frame:SetScript("OnMouseDown", function() frame:StartMoving() end)
    --frame:SetScript("OnMouseUp", function() frame:StopMovingOrSizing() end)
    --frame:SetScript("OnHide",frameOnClose)
    --#endregion

	
	-- Create our title frame
    local titleFrame = CreateFrame("Frame", nil, frame)
    --#region Setup main frame title
    titleFrame:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    titleFrame:SetBackdropColor(0,0,0,1)
    titleFrame:SetHeight(22)
    titleFrame:EnableMouse()
    titleFrame:EnableMouseWheel(true)
    titleFrame:SetResizable()
    titleFrame:SetMovable(true)
    titleFrame:SetPoint("LEFT",frame,"TOPLEFT",20,0)
    titleFrame:SetPoint("RIGHT",frame,"TOPRIGHT",-20,0)

    titleFrame:SetScript("OnMouseDown", function() frame:StartMoving() end)
    titleFrame:SetScript("OnMouseUp", function() frame:StopMovingOrSizing() end)
    titleFrame:SetScript("OnMouseWheel", function(s, delta)
      self:SetUIScale( max(min(frame:GetScale(0.8) + delta/15,5.0),0.5) );
    end)
	
	-- Create the text to go into the title frame
    local titletext = titleFrame:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
    titletext:SetPoint("CENTER",titleFrame,"CENTER",0,1)
	titletext:SetText ( "Vita Planner by Azulorr <Vita Obscura> - Caelestrasz US-Oceanic" )
    frame.titleFrame = titleFrame

	-- Add our planner frame to the addon globally
    self.frame = frame;
	-- Return the new frame
	return self.frame
end

function VitaPlannerL:CreatePlannerLeaderFrame()
	-- If we don't have our enclosing frame, then make one
	if not self.frame then self:InitUI() end;
	
	-- Create our surrounding frame to hold all of out details 
	local plannerFrame = CreateFrame("Frame", nil, self.frame)
    plannerFrame:Show()
	-- Take a slightly thinner width for effect
	plannerFrame:SetWidth(self.frame:GetWidth() - 20)
	-- Take the same height of the parent frame
    plannerFrame:SetHeight(self.frame:GetHeight() - 10)
    --[[plannerFrame:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
    plannerFrame:SetBackdropColor(0.1,0.1,0.1,0.8)--]]
    plannerFrame:SetPoint("LEFT",self.frame,"LEFT",20,0)
    plannerFrame:SetPoint("RIGHT",self.frame,"RIGHT",-20,0)
	
	-- Create the boss info (top frame) for finer configuration
	local bossInfo = CreateFrame("Frame", nil, plannerFrame)
	bossInfo:Show()
	bossInfo:SetWidth(600)
	bossInfo:SetHeight(160)
	bossInfo:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
	bossInfo:SetBackdropColor(0.12,0.12,0.12,0.6)
	bossInfo:SetPoint("TOP", plannerFrame, "TOP", 0, -10)
	bossInfo:SetPoint("CENTER", plannerFrame, "CENTER", 0, 0)
	plannerFrame.bossInfo = bossInfo
	
	-- Create the raid info (bottom frame) for raid / selection displays
	local raidInfo = CreateFrame("Frame", nil, plannerFrame)
	raidInfo:Hide()
	raidInfo:SetWidth(600)
	raidInfo:SetHeight(370)
	raidInfo:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
	raidInfo:SetBackdropColor(0.12,0.12,0.12,0.6)
	raidInfo:SetPoint("TOP", bossInfo, "BOTTOM", 0, -10)
	raidInfo:SetPoint("CENTER", plannerFrame, "CENTER", 0, 0)
	
	-- Create the "In" list
	local infoInList = CreateFrame("Frame", nil, raidInfo)
	infoInList:SetWidth(300)
	infoInList:SetHeight(370)
	infoInList:SetPoint("TOPLEFT", raidInfo, "TOPLEFT", 0, 0)
	raidInfo.infoInList = infoInList
	
	-- Create the "In" header
	local infoInHeader = infoInList:CreateFontString(nil,"OVERLAY","GameFontNormal")
	infoInHeader:SetText("In")
	infoInHeader:SetPoint("TOPLEFT", infoInList, "TOPLEFT", 10, -10)
	
	-- Create the "In" text 
	local infoIn = infoInList:CreateFontString(nil,"OVERLAY","GameFontNormal")
	infoIn:SetPoint("TOPLEFT", infoInHeader, "TOPLEFT", 0, -10)
	self.infoInString = infoIn
	
	-- Create the "Out" list
	local infoOutList = CreateFrame("Frame", nil, raidInfo)
	infoOutList:SetWidth(300)
	infoOutList:SetHeight(370)
	infoOutList:SetPoint("TOPRIGHT", raidInfo, "TOPRIGHT", 0, 0)
	raidInfo.infoOutList = infoOutList
	
	-- Create the "Out" header
	local infoOutHeader = infoOutList:CreateFontString(nil,"OVERLAY","GameFontNormal")
	infoOutHeader:SetText("Out")
	infoOutHeader:SetPoint("TOPLEFT", infoOutList, "TOPLEFT", 10, -10)
	
	-- Create the "Out" text
	local infoOut = infoOutList:CreateFontString(nil,"OVERLAY","GameFontNormal")
	infoOut:SetPoint("TOPLEFT", infoOutHeader, "TOPLEFT", 0, -10)
	self.infoOutString = infoOut
	
	-- Add the raid info to the surrounding frame
	plannerFrame.raidInfo = raidInfo
	
	-- Duplicate the raidInfo frame and put it in the same place
	local raidInfo2 = CreateFrame("Frame", nil, plannerFrame)
	-- Hide this one by default
	raidInfo2:Show()
	raidInfo2:SetWidth(600)
	raidInfo2:SetHeight(370)
	raidInfo2:SetBackdrop({
      bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 12,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
	-- Bit more of a red background
	raidInfo2:SetBackdropColor(0.12,0.12,0.12,0.6)
	raidInfo2:SetPoint("TOP", bossInfo, "BOTTOM", 0, -10)
	--raidInfo2:SetPoint("CENTER", plannerFrame, "CENTER", 0, 0)
	
	raidInfo2.group = {}

	local raidGroupsFrame = CreateFrame("Frame", nil, raidInfo2)
	local groupFramesHeight = (RAID_GROUP_VERT_GAP - 20) + ((RAID_UNIT_HEIGHT * 5) + (RAID_UNIT_VERT_GAP * 4)) + RAID_GROUP_VERT_GAP + ((RAID_UNIT_HEIGHT * 5) + (RAID_UNIT_VERT_GAP * 4))
	--print(groupFramesHeight)
	raidGroupsFrame:SetHeight(groupFramesHeight)
	local groupFramesWidth = (RAID_UNIT_WIDTH * 5) + (RAID_GROUP_HORZ_GAP * 4)
	raidGroupsFrame:SetWidth(groupFramesWidth)
	raidGroupsFrame:ClearAllPoints()
	raidGroupsFrame:SetPoint("CENTER", raidInfo2, "CENTER", 0, 0)
	
	local lastGroupFrame = nil
	for i=1, 8 do
		local groupFrame = self:CreateGroupFrame(raidGroupsFrame, lastGroupFrame, i)
		raidInfo2.group[i] = groupFrame
		lastGroupFrame = groupFrame
	end
	
	-- Populate the boss info frame
	self:PopulateBossInfoPanel(plannerFrame.bossInfo)
	-- Populate the raid display
	self:UpdateRaidInfoGroups()
	
	raidInfo2.raid = raidGroupsFrame
	plannerFrame.raidInfo2 = raidInfo2
	
	local finaliseBtn = CreateFrame("Button",nil,raidInfo2,"UIPanelButtonTemplate")
	finaliseBtn:Show()
	finaliseBtn:SetText ("Announce Groups Set")
	finaliseBtn:SetWidth(finaliseBtn:GetFontString():GetWidth() + 20)
	--finaliseBtn:SetTextFontObject("GameFontNormal")
	finaliseBtn:SetPoint("TOPLEFT",_G["VitaGroup_8"],"TOPRIGHT",10,0)
	finaliseBtn:SetScript("OnClick", function()
		--print("Sending Groups Set")
		local message, maxGroupNum = self:CreateGroupSetMessage()
		if IsInRaid() then
			self:SendClientCommand("SET_GROUPS", message, "RAID")
		else
			self:SendClientCommand("SET_GROUPS", message, UnitName("player"))
		end
		
		SendChatMessage( MSG_Prefix .. "Groups have been set! Please check 1 to "..maxGroupNum, "RAID_WARNING")
	end)
	
	
	-- Return the base frame
    return plannerFrame
end

function VitaPlannerL:CreateGroupSetMessage()
	local MAX_NUM_RAIDERS = 40
	local numRaidMembers = GetNumGroupMembers();
	
	--[[ DIFFICULTY IDS
		{	["NAME"] = "Flexible",		["VALUE"] = 14 },
		{	["NAME"] = "10 Normal",		["VALUE"] = 3 },
		{	["NAME"] = "25 Normal",		["VALUE"] = 4 },
		{	["NAME"] = "10 Heroic",		["VALUE"] = 5 },
		{	["NAME"] = "25 Heroic",		["VALUE"] = 6 }
	--]]
	self:UpdateMaxGroupSet()
	
	local message = self.selectedBoss .. "^" .. maxSetNum .. "^"
	local raiderList = {}
	for i = 1, MAX_NUM_RAIDERS do
		if IsInRaid() and i <= numRaidMembers then
			local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, loot = GetRaidRosterInfo(i);
			local person = format("%s;%d", name, subgroup)
			tinsert(raiderList, person)
		end			
	end
	
	message = message .. strjoin('*', unpack(raiderList))
	--print(message)
	return message, maxSetNum
end

function VitaPlannerL:UpdateRaidInfoGroups()
	if not self.frame then return end

	local MAX_NUM_RAIDERS = 40
	local numRaidMembers = GetNumGroupMembers();
	local groups = { [1] = {}, [2] = {}, [3] = {}, [4] = {}, [5] = {}, [6] = {}, [7] = {}, [8] = {} }
	
	for i = 1, MAX_NUM_RAIDERS do
		if IsInRaid() and i <= numRaidMembers then
			--local name = _G["RaidGroupButton"..i.."Name"];
			local unit = "raid"..i
			local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, loot = GetRaidRosterInfo(i);
			--print(unit .. " - " .. name .. " : " .. subgroup)
			-- Check party members
			local specrole = ""
			--[[if VitaPlanner.UnitIsUnit(unit,"player") then
				unit = "player"
			end--]]
			if self.Raiders[name] then
				specrole = self.Raiders[name].SpecRole
			end
			local info = { ["NAME"] = name, ["ID"] = i, ["GROUP"] = subgroup, ["SPECROLE"] = specrole, ["CLASS"] = class, ["ONLINE"] = online }
			tinsert(groups[subgroup], info)				
		end
	end
	
	-- Set the text of all raid unit infos
	--local totalCounter = 1
	for i = 1, 8 do
		local maxGUnits = 5
		for count = 1, maxGUnits do
			local newText = ""
			local raidIndex = count + ((tonumber(i)-1) * 5)
			--Default the backdrop
			_G["VitaRaidUnit_"..raidIndex]:SetBackdropColor(0.1,0.1,0.1,0.8)
			--Default the icon
			_G["VitaRaidUnit_"..raidIndex].icon:SetTexture(0,0,0,0)
			if groups[i][count] then
				-- Get the name of the raider and assign some special globals for identification
				newText = groups[i][count].NAME
				_G["VitaRaidUnit_"..raidIndex.."-NUMBER"] = groups[i][count].ID
				_G["VitaRaidUnit_"..raidIndex.."-SPECROLE"] = groups[i][count].SPECROLE
				_G["VitaRaidUnit_"..raidIndex.."-CLASS"] = groups[i][count].CLASS
				_G["VitaRaidUnit_"..raidIndex.."-ONLINE"] = groups[i][count].ONLINE
				_G["VitaRaidSpecialIndex_"..groups[i][count].ID] = _G["VitaRaidUnit_"..raidIndex]
				_G["VitaRaidSpecialIndex_"..groups[i][count].ID].icon:SetAlpha(0.5)
				--totalCounter = totalCounter + 1
				if groups[i][count].SPECROLE then
					if groups[i][count].SPECROLE ~= "" then
						--print (raidIndex .. " - ID: " .. groups[i][count].ID .. " - Name: " .. groups[i][count].NAME .. " - Role: " .. groups[i][count].SPECROLE)
						_G["VitaRaidSpecialIndex_"..groups[i][count].ID].icon:SetTexture("Interface\\Addons\\VitaPlanner_Leader\\icons\\"..strlower(groups[i][count].SPECROLE)..".tga")		
					end
				end
			end
			-- Assign the proper group position to this name
			_G["VitaGroup_"..i.."_Unit_"..count].name:SetText(newText)			
		end
	end
	
	
	
end

function VitaPlannerL:CreateGroupFrame(raidInfoFrame, groupFrame, index)

	local frame = CreateFrame("Frame", "VitaGroup_"..index, raidInfoFrame)
	if not groupFrame then
		frame:SetPoint("TOPLEFT", raidInfoFrame, "TOPLEFT", 0, -(RAID_GROUP_VERT_GAP-20))
	elseif index == 6 then
		frame:SetPoint("TOPLEFT", _G["VitaGroup_1"], "BOTTOMLEFT", 0, -RAID_GROUP_VERT_GAP)
	else
		frame:SetPoint("TOPLEFT", groupFrame, "TOPRIGHT", RAID_GROUP_HORZ_GAP, 0)
	end
	frame:SetWidth(RAID_UNIT_WIDTH)
	frame:SetHeight(RAID_UNIT_HEIGHT * 5 + RAID_UNIT_VERT_GAP * 4)
	frame:Show()
	frame:SetBackdrop({
      --bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
      edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
      tile = true, tileSize = 64, edgeSize = 6,
      insets = { left = 2, right = 1, top = 2, bottom = 2 }
    })
	
	local frameName = frame:CreateFontString(nil,"OVERLAY","GameFontNormal")
	frameName:SetText("Group "..index)
	frameName:SetPoint("BOTTOM", frame, "TOP", 0, 5)
	frameName:Show()
	
	frame.units = {}
	
	for i=1, 5 do
		local raidIndex = i + ((tonumber(index)-1) * 5)
		local unitPosition = CreateFrame("Frame", "VitaRaidUnit_"..raidIndex, frame)
		unitPosition:SetHeight(RAID_UNIT_HEIGHT)
		if i == 1 then
			unitPosition:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, 0)
		else
			unitPosition:SetPoint("TOPLEFT", frame.units[i-1], "BOTTOMLEFT", 0, -2)
		end
		unitPosition:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
		unitPosition:SetBackdrop({
			bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true, tileSize = 64, edgeSize = 6,
			insets = { left = 2, right = 1, top = 2, bottom = 2 }
		})
		unitPosition:EnableMouse()
		unitPosition:RegisterForDrag("LeftButton")
		unitPosition:SetScript("OnDragStart", function(frame, button) self:BeginMovingRaidMember(frame,button) end)
		unitPosition:SetScript("OnDragStop", function(frame) self:TryInitiateSwap(frame) end)
		
		unitPosition:SetBackdropColor(0.1,0.1,0.1,0.8)
		
		local unitFrame = CreateFrame("Frame", "VitaGroup_"..index.."_Unit_"..i, unitPosition)
		unitFrame:SetHeight(RAID_UNIT_HEIGHT)
		unitFrame:SetPoint("TOPLEFT", unitPosition, "TOPLEFT", 0, 0)
		unitFrame:SetPoint("BOTTOMRIGHT", unitPosition, "BOTTOMRIGHT", 0, 0)
		
		local unitName = unitFrame:CreateFontString("VitaGroup_"..index.."_Unit_"..i.."_NAME", "OVERLAY", "GameFontNormalSmall")
		unitName:SetText("")
		unitName:SetWidth(RAID_UNIT_WIDTH-RAID_UNIT_HEIGHT-5)
		unitName:SetWordWrap(false)
		unitName:SetJustifyH("LEFT")
		unitName:SetPoint("LEFT", unitFrame, "LEFT", 5, 0)
		unitFrame.name = unitName
		
		local unitSpec = CreateFrame("Frame",nil,unitFrame)
		unitSpec:SetWidth(RAID_UNIT_HEIGHT-5)
		unitSpec:SetHeight(RAID_UNIT_HEIGHT-5)
		unitSpec:SetPoint("CENTER",unitFrame,"CENTER",0,0)
		unitSpec:SetPoint("RIGHT",unitFrame,"RIGHT",-5,0)
		local unitSpecTex = unitSpec:CreateTexture()
		unitSpecTex:SetAllPoints()
		
		unitSpec.tex = unitSpecTex
		unitFrame.spec = unitSpec
		
		
		_G["VitaRaidUnit_"..raidIndex.."_NAME"] = unitName
		unitPosition.frame = unitFrame
		unitPosition.icon = unitFrame.spec.tex
		frame.units[i] = unitPosition
	end
	
	return frame
end

function VitaPlannerL:CopyRaidUnitFrame(frame)
	--local raidIndex = i + ((tonumber(index)-1) * 5)
	local unitPosition = CreateFrame("Frame", nil, UIParent) 		-- A.K.A: "VitaRaidUnit_"..raidIndex
	unitPosition:Hide()
	unitPosition:SetHeight(RAID_UNIT_HEIGHT)
	unitPosition:SetWidth(RAID_UNIT_WIDTH)
	--unitPosition:SetPoint("RIGHT", frame, "RIGHT", 0, 0)
	unitPosition:SetBackdrop({
		bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 64, edgeSize = 6,
		insets = { left = 2, right = 1, top = 2, bottom = 2 }
	})
	--unitPosition:EnableMouse()
	--unitPosition:RegisterForDrag("LeftButton")
	--unitPosition:SetScript("OnDragStart", function(frame, button) self:BeginMovingRaidMember(frame,button) end)
	--unitPosition:SetScript("OnDragStop", function(frame) self:TryInitiateSwap(frame) end)	
	unitPosition:SetBackdropColor(0.1,0.1,0.1,0.8)
	unitPosition:SetScript("OnUpdate", function(s, elapsed)
		local uiScale, cursorX, cursorY = UIParent:GetEffectiveScale(), GetCursorPosition()
		--print (cursorX .. " , " .. cursorY)
		s:SetPoint("CENTER", nil, "BOTTOMLEFT",cursorX / uiScale,cursorY / uiScale)
	end)
	unitPosition:SetFrameStrata("TOOLTIP")
	unitPosition:SetAlpha(0.75)
	
	local unitFrame = CreateFrame("Frame", nil, unitPosition)		-- A.K.A: "VitaGroup_"..index.."_Unit_"..i
	unitFrame:SetHeight(RAID_UNIT_HEIGHT)
	unitFrame:SetPoint("TOPLEFT", unitPosition, "TOPLEFT", 0, 0)
	unitFrame:SetPoint("BOTTOMRIGHT", unitPosition, "BOTTOMRIGHT", 0, 0)
	
	local unitName = unitFrame:CreateFontString(nil, "OVERLAY", "GameFontNormalSmall")		-- A.K.A: "VitaGroup_"..index.."_Unit_"..i.."_NAME"
	unitName:SetText("")
	unitName:SetWidth(RAID_UNIT_WIDTH-RAID_UNIT_HEIGHT-5)
	unitName:SetWordWrap(false)
	unitName:SetJustifyH("LEFT")
	unitName:SetPoint("LEFT", unitFrame, "LEFT", 5, 0)
	unitFrame.name = unitName
	
	local unitSpec = CreateFrame("Frame",nil,unitFrame)
	unitSpec:SetWidth(RAID_UNIT_HEIGHT-5)
	unitSpec:SetHeight(RAID_UNIT_HEIGHT-5)
	unitSpec:SetPoint("CENTER",unitFrame,"CENTER",0,0)
	unitSpec:SetPoint("RIGHT",unitFrame,"RIGHT",-5,0)
	local unitSpecTex = unitSpec:CreateTexture()
	unitSpecTex:SetAllPoints()
	unitSpecTex:SetAlpha(0.5)
	
	unitSpec.tex = unitSpecTex
	unitFrame.spec = unitSpec
	unitPosition.frame = unitFrame	
	
	return unitPosition
end

function VitaPlannerL:BeginMovingRaidMember(frame, button)
	--print ("Drag started on: "..frame:GetName())
	self.currentDraggedUnitFrame = frame
	local draggedFrame = self.draggedFrameVisual
	if not draggedFrame then
		print("generating dragged frame")
		draggedFrame = self:CopyRaidUnitFrame(self.currentDraggedUnitFrame:GetName())
		self.draggedFrameVisual = draggedFrame
	end
	
	draggedFrame.frame.name:SetText(_G[frame:GetName().."_NAME"]:GetText())
	draggedFrame.frame.spec.tex:SetTexture(_G[frame:GetName()].icon:GetTexture())
	draggedFrame:Show()
	--print (self.currentDraggedUnitFrame:GetName())
end

function VitaPlannerL:TryInitiateSwap(frame)
	if (string.match(GetMouseFocus():GetName(),"VitaRaidUnit")) then
		local _, initID = strsplit("_", self.currentDraggedUnitFrame:GetName())
		local actualInitID = _G["VitaRaidUnit_"..initID.."-NUMBER"]
		--print("initID "..initID.." - actualID "..actualInitID)
		local _, targetID = strsplit("_", GetMouseFocus():GetName())
		
		--print(_G["VitaRaidUnit_"..targetID.."_NAME"]:GetText())
		if _G["VitaRaidUnit_"..targetID.."_NAME"]:GetText() then
			local actualTargetID = _G["VitaRaidUnit_"..targetID.."-NUMBER"]
			--print("targetID "..targetID.." - actualID "..actualTargetID)
			SwapRaidSubgroup(tonumber(actualInitID), tonumber(actualTargetID))
		else
			local _, targetGroup = strsplit("_", GetMouseFocus():GetParent():GetName())
			--print("Target Group: "..targetGroup)
			SetRaidSubgroup(tonumber(actualInitID), tonumber(targetGroup))
		end
	end
	self.draggedFrameVisual:Hide()
end

function VitaPlannerL:UpdateMaxGroupSet()
	if self.difficulty == 3 or self.difficulty == 5 then
		maxSetNum = 2
	elseif self.difficulty == 4 or self.difficulty == 6 then
		maxSetNum = 5
	else
		maxSetNum = 6
	end
end

function VitaPlannerL:UpdateLists()
	self:UpdateInList()
	self:UpdateOutList()
	self:UpdateIgnoreList()	
end

function VitaPlannerL:UpdateInList()
    local realmName = "-"..GetRealmName();
    local ret = ""
    
    if self.inList == nil then return ret end
    
    for key,value in pairs(self.inList) do 
        
        local player = string.gsub(key, realmName, "");
        
        local groupNumber = 40;
        local raidIndex = UnitInRaid(player);
        
        if (raidIndex ~= nil) then
            _,_, groupNumber = GetRaidRosterInfo(raidIndex);
        else
			return
		end        
		
		if groupNumber <= maxSetNum then
			-- Colour this unit green
			--print(player .. " - " .. raidIndex)
			_G["VitaRaidSpecialIndex_"..raidIndex]:SetBackdropColor(0,0.5,0,0.4)
		end
		if groupNumber >= maxSetNum+1 then
			-- Colour this unit red
			_G["VitaRaidSpecialIndex_"..raidIndex]:SetBackdropColor(0.5,0,0,0.4)
		end

    end

end

function VitaPlannerL:UpdateOutList()
    local realmName = "-"..GetRealmName();
    local ret = ""
    
    if self.outList == nil then return ret end
    
    for key,value in pairs(self.outList) do 
        
        local player = string.gsub(key, realmName, "");
        
        local groupNumber = 40;
        local raidIndex = UnitInRaid(player);
        
        if (raidIndex ~= nil) then
            _,_, groupNumber = GetRaidRosterInfo(raidIndex);
        else
			return
		end        
		
		if groupNumber <= maxSetNum then
			-- Colour this unit red
			_G["VitaRaidSpecialIndex_"..raidIndex]:SetBackdropColor(0.5,0,0,0.4)
		end
		if groupNumber >= maxSetNum+1 then
			-- Colour this unit green
			_G["VitaRaidSpecialIndex_"..raidIndex]:SetBackdropColor(0,0.5,0,0.4)
		end

    end

end

function VitaPlannerL:UpdateIgnoreList()
    local realmName = "-"..GetRealmName();
    local ret = ""
    
    if self.ignoreList == nil then return ret end
    
    for key,value in pairs(self.ignoreList) do 
        
        local player = string.gsub(key, realmName, "");
        
        local groupNumber = 40;
        local raidIndex = UnitInRaid(player);
        
        if (raidIndex ~= nil) then
            _,_, groupNumber = GetRaidRosterInfo(raidIndex);
        else
			return
		end
		
		-- Colour this unit yellow
		_G["VitaRaidSpecialIndex_"..raidIndex]:SetBackdropColor(0.5,0.5,0,0.4)

    end
    
end


function VitaPlannerL:PopulateBossInfoPanel(bossInfo)
	-- Raid Zone Select
	local height = 0
	
	local zoneText = bossInfo:CreateFontString(nil,"OVERLAY","GameFontNormal")
	zoneText:SetText("Raid")
	
	local zoneSelect = CreateFrame("Frame", "ZoneSelectDropDown", bossInfo, "UIDropDownMenuTemplate")
		
	zoneSelect:SetPoint("LEFT", zoneText, "LEFT", 200, -5)
	UIDropDownMenu_Initialize(zoneSelect, function(...) VitaPlannerL.ZoneSelectDropDownInitialize(VitaPlannerL, ...) end);	
	UIDropDownMenu_SetWidth(zoneSelect, 150)
	UIDropDownMenu_SetButtonWidth(zoneSelect, 174)
	UIDropDownMenu_SetText(zoneSelect, self.zoneName)
	self.ZoneSelect = zoneSelect
	zoneSelect:Show()
	ToggleDropDownMenu(nil, nil, zoneSelect)
	
	height = height + zoneText:GetHeight()
	
	-- Raid Difficulty Select
	local difficultyText = bossInfo:CreateFontString(nil,"OVERLAY","GameFontNormal")
	difficultyText:SetPoint("TOPLEFT", zoneText, "BOTTOMLEFT", 0, -20)
	difficultyText:SetText("Raid Difficulty")
	
	height = height + 20
	
	local difficultySelect = CreateFrame("Frame", "DifficultySelectDropDown", bossInfo, "UIDropDownMenuTemplate")
		
	difficultySelect:SetPoint("LEFT", difficultyText, "LEFT", 200, -5)
	UIDropDownMenu_Initialize(difficultySelect, function(...) VitaPlannerL.DifficultySelectDropDownInitialize(VitaPlannerL, ...) end);	
	UIDropDownMenu_SetWidth(difficultySelect, 150)
	UIDropDownMenu_SetButtonWidth(difficultySelect, 174)
	UIDropDownMenu_SetText(difficultySelect, self.diffName)	
	self.DifficultySelect = difficultySelect
	difficultySelect:Show()
	ToggleDropDownMenu(nil, nil, difficultySelect)
	
	height = height + difficultyText:GetHeight()
	
	-- Raid Boss Select
	local bossText = bossInfo:CreateFontString(nil,"OVERLAY","GameFontNormal")
	bossText:SetPoint("TOPLEFT", difficultyText, "BOTTOMLEFT", 0, -20)
	bossText:SetText("Boss")
	
	height = height + 20
	
	local bossSelect = CreateFrame("Frame", "BossSelectDropDown", bossInfo, "UIDropDownMenuTemplate")
		
	bossSelect:SetPoint("LEFT", bossText, "LEFT", 200, -5)
	UIDropDownMenu_Initialize(bossSelect, function(...) VitaPlannerL.BossSelectDropDownInitialize(VitaPlannerL, ...) end);	
	UIDropDownMenu_SetWidth(bossSelect, 150)
	UIDropDownMenu_SetButtonWidth(bossSelect, 174)
	UIDropDownMenu_SetText(bossSelect, "Select Boss")
	ToggleDropDownMenu(nil, nil, bossSelect)
	self.BossSelect = bossSelect
	
	height = height + bossText:GetHeight()
	height = height / 2
	zoneText:SetPoint("CENTER", bossInfo, "CENTER", 0, 0)
	zoneText:SetPoint("LEFT", bossInfo, "LEFT", 15, height)
	
	-- Check Raid Button
	local checkRaid = CreateFrame("Button", nil, bossInfo, "UIPanelButtonTemplate")
	checkRaid:Show()	
	checkRaid:SetText("Check Raid")
	checkRaid:SetWidth(checkRaid:GetFontString():GetWidth() + 20)
	checkRaid:SetPoint("TOPRIGHT", bossInfo, "TOPRIGHT", -10, -10)
	checkRaid:SetScript("OnClick", function()
		self:AskRaidForSelections()
	end)
	bossInfo.checkRaid = checkRaid
	checkRaid:Disable()

	-- Create our close button
	local closeBtn = CreateFrame("Button", nil, bossInfo, "UIPanelButtonTemplate")
	closeBtn:Show()	
	--closeBtn:SetHeight(20)
	closeBtn:SetText("Close")
	closeBtn:SetWidth(closeBtn:GetFontString():GetWidth() + 20)
	closeBtn:SetPoint("BOTTOMRIGHT", bossInfo, "BOTTOMRIGHT", -10, 10)
	closeBtn:SetScript("OnClick", function()
										self:Hide()
									end)
	bossInfo.closeBtn = closeBtn
	
	-- Cancel Check Button
	local cancelCheckRaid = CreateFrame("Button", nil, bossInfo, "UIPanelButtonTemplate")
	cancelCheckRaid:Hide()
	cancelCheckRaid:SetText("Reset")
	cancelCheckRaid:SetWidth(cancelCheckRaid:GetFontString():GetWidth() + 20)
	cancelCheckRaid:SetPoint("BOTTOMRIGHT", closeBtn, "TOPRIGHT", 0, 10)
	cancelCheckRaid:SetScript("OnClick", function()
		self.timeout = 0
		self.timeoutLeft = 0
		self.mainFrame.bossInfo.checkRaid:Enable()
		self.inList = {}
		self.outList = {}
		self.ignoreList = {}
		self:UpdateMaxGroupSet()
		self:UpdateRaidInfoGroups()
		self:UpdateLists()
		self.cancelCheckRaid:Hide()
	end)
	self.cancelCheckRaid = cancelCheckRaid
	
	--self.checkTimerFrame = VitaPlanner:GenerateTimerFrame()
	--self.checkTimerFrame:Hide()
	--timerFrame:Hide()	
end

function VitaPlannerL:DifficultySelectDropDownInitialize( frame, level, menuList )
	if not VitaPlannerL.DifficultySelect then return end
	
	local info = UIDropDownMenu_CreateInfo()
	
	info.text = "Raid Difficulty"
	info.disabled = true
	info.isTitle = true;
	info.notClickable = 1
	info.notCheckable = 1
	info.tooltipTitle = nil
	info.tooltipText = nil
	UIDropDownMenu_AddButton(info,UIDROPDOWNMENU_MENU_LEVEL)
	
	for i, d in ipairs(self.DIFFICULTIES) do
		info = UIDropDownMenu_CreateInfo()
		info.text = d.NAME
		info.value = d.VALUE
		info.func = function()
			self.difficulty = d.VALUE
			self.diffName = d.NAME
			UIDropDownMenu_SetSelectedID(frame, i+1)
			UIDropDownMenu_SetText(frame, d.NAME)
			--VitaPlannerL.ModifyRaidInfoLayout() =NYI=
			self:UpdateMaxGroupSet()
			self:UpdateLists()
		end		
		info.checked = function() if self.difficulty == d.VALUE then return true end end
		UIDropDownMenu_AddButton(info, level)
	end
end

function VitaPlannerL:ZoneSelectDropDownInitialize( frame, level, menuList )
	if not VitaPlannerL.ZoneSelect then return end
	
	local info = UIDropDownMenu_CreateInfo()
	
	info.text = "Raid"
	info.disabled = true
	info.isTitle = true;
	info.notClickable = 1
	info.notCheckable = 1
	info.tooltipTitle = nil
	info.tooltipText = nil
	UIDropDownMenu_AddButton(info,UIDROPDOWNMENU_MENU_LEVEL)
		
	for i,raid in ipairs(self.BOSSES) do
		info = UIDropDownMenu_CreateInfo()
		info.text = raid.NAME
		info.value = raid.ABBR
		info.func = function() 
			self.zone = raid.INSTANCEID 
			self.zoneName = raid.NAME
			self.selectedRaid = raid.ABBR
			self.selectedBossList = raid.BOSS
			UIDropDownMenu_SetSelectedID(frame, i+1)
			UIDropDownMenu_SetText(frame, raid.NAME)
		end
		info.checked = function() if self.selectedRaid == raid.ABBR then return true end end
		UIDropDownMenu_AddButton(info, level)
	end
	
end

function VitaPlannerL:BossSelectDropDownInitialize( frame, level, menuList )
	if not VitaPlannerL.BossSelect then return end
	
	local info = UIDropDownMenu_CreateInfo()
	
	info.text = "Boss"
	info.disabled = true
	info.isTitle = true;
	info.notClickable = 1
	info.notCheckable = 1
	info.tooltipTitle = nil
	info.tooltipText = nil
	UIDropDownMenu_AddButton(info,UIDROPDOWNMENU_MENU_LEVEL)
	
	if not self.selectedBossList then return end
	
	for i,boss in ipairs(self.selectedBossList) do	
		info = UIDropDownMenu_CreateInfo()
		info.text = boss.NAME
		info.value = boss.ENCOUNTERID
		info.func = function() 
			self.selectedBossID = boss.ENCOUNTERID
			self.selectedBoss = boss.NAME
			self.mainFrame.bossInfo.checkRaid:Enable()
			UIDropDownMenu_SetSelectedID(frame, i+1)
			UIDropDownMenu_SetText(frame, boss.NAME)
			end
		UIDropDownMenu_AddButton(info, level)
	end
end

function VitaPlannerL:Debug( message, verbose )
    if not self.debug then return end;
    if verbose and not self.verbose then return end;
    self:Print("debug: " .. message)
end

function VitaPlannerL:SendClientCommand(command, message, target)
	if not target then
		return self:Print(L["Could not send command, no target specified"])
	end;
    if target=='RAID' then
        self:SendCommMessage("VitaPlannerC", format("%s:%s", tostring(command), tostring(message)), "RAID", nil, "ALERT")
    elseif target=='PARTY' then
        self:SendCommMessage("VitaPlannerC", format("%s:%s", tostring(command), tostring(message)), "PARTY", nil, "ALERT")
    elseif target=='GUILD' then
        self:SendCommMessage("VitaPlannerC", format("%s:%s", tostring(command), tostring(message)), "GUILD", nil, "ALERT")
    else
        -- Don't use AceComm for messages to self, call function directly
        if VitaPlanner.UnitIsUnit(target, 'player') then
			--print("Target is me")
            VitaPlanner.CommandReceived(VitaPlanner, "VitaPlannerC", format("%s:%s", tostring(command), tostring(message)), 'WHISPER', target)
        else
            self:SendCommMessage("VitaPlannerC", format("%s:%s", tostring(command), tostring(message)), "WHISPER", target, "ALERT")
        end
    end
end

function VitaPlannerL:SendLeaderCommand(command, message, target)
	if not target then
		return self:Print(L["Could not send command, no target specified"])
	end;
    if target=='RAID' then
        self:SendCommMessage("VitaPlannerL", format("%s:%s", tostring(command), tostring(message)), "RAID", nil, "ALERT")
    elseif target=='PARTY' then
        self:SendCommMessage("VitaPlannerL", format("%s:%s", tostring(command), tostring(message)), "PARTY", nil, "ALERT")
    elseif target=='GUILD' then
        self:SendCommMessage("VitaPlannerL", format("%s:%s", tostring(command), tostring(message)), "GUILD", nil, "ALERT")
    else
        -- Don't use AceComm for messages to self, call function directly
        if VitaPlanner.UnitIsUnit(target, 'player') then
			--print("Target is me")
            VitaPlanner.CommandReceived(VitaPlanner, "VitaPlannerL", format("%s:%s", tostring(command), tostring(message)), 'WHISPER', target)
        else
            self:SendCommMessage("VitaPlannerL", format("%s:%s", tostring(command), tostring(message)), "WHISPER", target, "ALERT")
        end
    end
end

function VitaPlannerL:CommandReceived(prefix, message, distribution, sender)
	local _,_,command, message = string.find(message, "^([%a_]-):(.*)$")
	command = strupper(command or '');
	message = message or '';

	sender = VitaPlanner.UnAmbiguate(sender)
	
	--print (message)
	
	if command == "SELECTION" then
	
		local pFrame = self.mainFrame
        if not pFrame then
            pFrame = self:CreatePlannerLeaderFrame()			
            self.mainFrame = pFrame
        end
		
		-- Get the selection response from the sender
		local bossName, value = strsplit("^", message)
		--print(sender .. " - " .. strupper(value))
		value = strupper(value)
		if value == "IN" then
			self.inList[sender] = "yes"
			self.outList[sender] = nil
			--self.ignoreList[sender] = nil
		end
		
		if value == "OUT" then
			self.inList[sender] = nil
			self.outList[sender] = "yes"
			--self.ignoreList[sender] = nil
		end
		
		if value == "NONE" then
			self.inList[sender] = nil
			self.outList[sender] = nil
			--self.ignoreList[sender] = "yes"
		end
		
		-- Update the selection list visuals
		self:UpdateMaxGroupSet()
		self:UpdateLists()
		
		-- TODO: Share the in/out lists to other raid members with assist or leader
		--self:ShareLists()
		if IsInRaid() and (UnitIsGroupAssistant("player") or UnitIsGroupLeader("player")) and VitaPlanner.UnitIsUnit(sender, "player") then
			-- Show the inner frames
			pFrame:Show()
			-- Show the outer frames
			self:Show()
		end
	elseif command == "RESET_LISTS" then
		if VitaPlanner.UnitIsUnit(sender, "player") then return end
		if IsInRaid() and (UnitIsGroupAssistant(sender) or UnitIsGroupLeader(sender)) then
			self:ResetLists()
		end
	elseif command == "COPY_WHISPER" then
		local sender, selection = strsplit("^",message)
		if selection == "IN" then
			self.inList[sender] = "yes"
			self.outList[sender] = nil
			--self.ignoreList[sender] = nil
		end		
		if selection == "OUT" then
			self.inList[sender] = nil
			self.outList[sender] = "yes"
			--self.ignoreList[sender] = nil
		end		
		if selection == "CANCEL" then
			self.inList[sender] = nil
			self.outList[sender] = nil
			--self.ignoreList[sender] = "yes"
		end			
	end
end

function VitaPlannerL:SendBossSelection( target, bossID, response )
    -- Just whisper the response back to the ml.
    self:SendCommand("SELECTION", format("%s^%s", bossID or 0, response or 0), target)
end

function VitaPlannerL:ResetLists()
	self.inList = {}
	self.outList = {}
	self.ignoreList = {}
	
	self:UpdateRaidInfoGroups()
	self:UpdateLists()
end

function VitaPlannerL:AskRaidForSelections()
	-- Ask the raid for their desired selection
	
	-- Create our buttons
	local numButtons = 2
	local buttons = {
		  {response = "IN",           text = "IN",				colour = 'ffffff'},
		  {response = "OUT",          text = "OUT",		colour = 'ffffff'}
		}
		
	local buttonString = {}		
	for i=1,numButtons do
		tinsert(buttonString,
			format('%s;%s;%s',
			   buttons[i].response,
			   buttons[i].text,
			   buttons[i].colour or 'ffffff'))
	end
	buttonString = strjoin('*', unpack(buttonString))
	--print (buttonString)
	
	-- Get the stored timeout value from our options
	local timeout = VitaPlannerL.db.profile.selection_timeout --10
	
	-- Get the raid, boss and difficulty information from our boss info panel
	local bossName = self.selectedBoss
	local bossID = self.selectedBossID
	local diffName = self.diffName
	local diffID = self.difficulty
	local raidID = self.zone
	
	-- Format our message
	local message = format('%d^%s^%d^%s^%d^%d^%s^%d',
							raidID,
							bossName,
							bossID,
							diffName,
							diffID,
							numButtons,
							buttonString,
							timeout)
	
	-- Reset the selection lists as we don't want old entries to taint the new selections
	self:ResetLists()
	self:SendLeaderCommand("RESET_LISTS", "IGNORE_MESSAGE", "RAID")
	
	-- Start our local timer (corresponds to the timeout given to the raid)
	-- TODO, make this work properly when hidden
	self.timeout = timeout
	self.timeoutLeft = self.timeout
	if self.timeout and self.timeout>0 then
		--self.checkTimerFrame.progressBar:SetMinMaxValues(0,self.timeout)
		--self.checkTimerFrame.progressBar:SetValue(self.timeoutLeft)
		--self.checkTimerFrame.lblTimeout:SetText('')
		--self.checkTimerFrame:Show()
		self.mainFrame.bossInfo.checkRaid:Disable()
		self.currentlyChecking = true
	else
		--self.checkTimerFrame:Hide()
		self.mainFrame.bossInfo.checkRaid:Enable()
		self.currentlyChecking = false
	end
	
	-- If we're in a raid, send the check to the raid
	if IsInRaid() then
		self:SendClientCommand("CHECK_BOSS", message, "RAID")
	--else
		-- If we're by ourself, ask ourself
		--VitaPlanner.CommandReceived(VitaPlannerC, 
		--self:SendClientCommand("CHECK_BOSS", message, UnitName("player"))
	end
	
	local journalLink = "|cff66bbff|Hjournal:1:" .. bossID .. ":" .. diffID .. "|h["..bossName.."]|h|r"
	local raidMessage = "Planning started for "..journalLink.." - Please whisper me with : #IN , #OUT or #CANCEL : within the next "..timeout.." seconds!"
	local raidMessage2 = "If you have the Vita Planner add-on installed, please use the buttons in the pop-up."
	SendChatMessage( MSG_Prefix .. raidMessage, "RAID_WARNING")
	SendChatMessage( MSG_Prefix .. raidMessage2, "RAID_WARNING")
end

function VitaPlannerL:RAID_GROUP_UPDATE(...)
	--print("Group updated")
	if not self.frame then return end
	LGIST:Rescan()
	--self:UpdateMaxGroupSet()
	self:UpdateRaidInfoGroups()
	self:UpdateLists()
end

function VitaPlannerL:HandleMSGCommand(command, message, sender, event)
	local senderUA = VitaPlanner.UnAmbiguate(sender)
	
	if self.currentlyChecking == false then
		self:SendWhisperResponse(UnitName("player") .. " is not currently accepting preferences.", senderUA)
		return true
	end
	--print(command .. " | " .. message .. " | " .. sender .. " | " .. event)
	if command == "IN" or command == "OUT" or command == "CANCEL" then
		local niceCommand = ""
		if command == "IN" then
			self.inList[sender] = "yes"
			self.outList[sender] = nil
			--self.ignoreList[sender] = nil		
			niceCommand = "In"
		elseif command == "OUT" then
			self.inList[sender] = nil
			self.outList[sender] = "yes"
			--self.ignoreList[sender] = nil
			niceCommand = "Out"
		elseif command == "CANCEL" then
			self.inList[sender] = nil
			self.outList[sender] = nil
			--self.ignoreList[sender] = "yes"
			niceCommand = "Cancel"
		end
	
		self:UpdateMaxGroupSet()
		-- Update the selection list visuals
		self:UpdateRaidInfoGroups()
		self:UpdateLists()
		self:SendLeaderCommand("COPY_WHISPER", format("%s^%s",sender,command), "RAID")
		
		local resMsg = niceCommand .. " received for " .. sender
		
		self:SendWhisperResponse(resMsg, senderUA)
		
		return true
	end
	return false
end

function VitaPlannerL:SendWhisperResponse(message, target)
    SendChatMessage( MSG_Prefix .. ( message or ''), 'WHISPER', nil, target );
    return false;
end

function VitaPlannerL:ChatFilter(...)
	local event = select(1, ...)
    local sender = VitaPlanner.UnAmbiguate(select(3, ...))
    local msg = select(2, ...)
    local msgID = select(12, ...)
	
	-- Do not process WIM History
    if not msgID or msgID<1 then return end
	
	if lastMsgID == msgID then
        return lastMsgFiltered
    end
	
	lastMsgID         = msgID
    lastMsgFiltered   = false
	
	local command, params = nil
	for i=1, numChatCommands do
		command = strmatch(msg, chatCommands[i])
		--print (command)
		--print ("===")
		if command == "[Vita Planner]" or command == "[VitaPlanner]" then return true end
		if command then break end
	end
	
	if command then
		local OK, ret = pcall(VitaPlannerL.HandleMSGCommand, VitaPlannerL, strupper(command or ''), strtrim(params or ''), sender, event)
		
		if not OK then
			print(format("Error parsing msg '%s' from %s: %s", tostring(params), tostring(sender), tostring(ret)))
		elseif OK and ret then
			--print("OK and ret true")
			lastMsgFiltered = true
		end

		return lastMsgFiltered
	end
end